"""A collection of templates and rendering functions."""
import subprocess


def get_ip_range(interface="docker0"):
    cmd = "ip address show dev " + interface
    addr = subprocess.check_output(cmd.split()).split()
    ip = addr[addr.index('inet') + 1].split('/')[0]
    return ip.split('.')[:-1] + ['0']


TEMPLATES = {"NGINX": {}, "BASE": {}}

TEMPLATES["BASE"]["CONTAINERS"] = """docker run -d -p {dockerdns}:53:53/udp --name skydns crosbymichael/skydns -nameserver {dns}:53 -domain drydock
docker run -d -v /var/run/docker.sock:/docker.sock --name skydock --dns {dockerdns} --link skydns:skydns crosbymichael/skydock -ttl 30 -environment containers -s /docker.sock -domain drydock
docker run -d -p 80:80 -p 443:443 --name nginx --dns {dockerdns} -v /etc/nginx/certs:/etc/nginx/certs -v /etc/nginx/sites-enabled:/etc/nginx/sites-enabled -v /var/log/nginx:/var/log/nginx dockerfile/nginx
cd /etc/nginx/certs && openssl genrsa -out server.key 2048 && openssl req -new -key server.key -out server.csr && openssl x509 -req -days 365 -in server.csr -signkey server.key -out server.crt"""

TEMPLATES["NGINX"]["HEADER"] = """events {
    worker_connections 1024;
    use epoll;
}"""

TEMPLATES["NGINX"]["UPSTREAM"] = """upstream {name} {{
    server {skyfqdn};
}}"""

TEMPLATES["NGINX"]["HTTP"] = """server {{
    listen       80;
    server_name  {fqdn};

    access_log  /var/log/nginx/{fqdn}.access.log  combined;
    error_log  /var/log/nginx/{fqdn}.error.log;

    resolver {dns} valid=5s;
    resolver_timeout 5s;

    location / {{
        {rules}
        proxy_set_header Host $host;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
        proxy_set_header X-NginX-Proxy true;
        proxy_redirect off;
        proxy_buffering off;

        proxy_pass http://{skyfqdn}:{port}/;
    }}
}}"""

TEMPLATES["NGINX"]["HTTPS"] = """server {{
    listen       443;
    server_name  {fqdn};

    access_log  /var/log/nginx/{fqdn}.access.log  combined;
    error_log  /var/log/nginx/{fqdn}.error.log;

    resolver {dns} valid=5s;
    resolver_timeout 5s;

    ssl on;
    ssl_session_timeout 5m;
    ssl_protocols SSLv2 SSLv3 TLSv1;
    ssl_ciphers ALL:!ADH:!EXPORT56:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv2:+EXP;
    ssl_prefer_server_ciphers on;
    ssl_certificate /etc/nginx/certs/server.crt;
    ssl_certificate_key /etc/nginx/certs/server.key;

    location / {{
        {rules}
        proxy_set_header Host $host;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
        proxy_set_header X-NginX-Proxy true;
        proxy_redirect off;
        proxy_buffering off;

        proxy_pass http://{skyfqdn}:{port}/;
    }}
}}"""


TEMPLATES["NGINX"]["RULES"] = """deny    {gateway};
        allow   {lan}/24;
        allow   {docker}/24;
        deny    all;
"""


NETWORK = {}


def prepare_networking(lani="eth0", dockeri="docker0",
                       gateway='1', dns="8.8.8.8"):
    """Stores useful network information and renders templates."""
    lanip = get_ip_range(lani)
    lan = '.'.join(lanip)
    gateway = '.'.join(lanip[:-1] + [gateway])
    dockerip = get_ip_range(dockeri)
    docker = '.'.join(dockerip)
    dockerdns = '.'.join(dockerip[:-1] + ['1'])

    TEMPLATES["NGINX"]["RULES"] = TEMPLATES["NGINX"]["RULES"].format(
        lan=lan, gateway=gateway, docker=docker)

    TEMPLATES["BASE"]["CONTAINERS"] = TEMPLATES["BASE"]["CONTAINERS"].format(
        lan=lan, gateway=gateway, docker=docker, dns=dns, dockerdns=dockerdns)
    NETWORK["lan"] = lan
    NETWORK["gateway"] = gateway
    NETWORK["docker"] = docker
    NETWORK["dockerdns"] = dockerdns
    NETWORK["dns"] = dns


def render_nginx_config(container):
    """
    Render the nginx reverse proxy configuration for the given container.
    """
    if not container.http_port and not container.https_port:
        return ""

    config = []

    if not container.external:
        rules = TEMPLATES["NGINX"]["RULES"]
    else:
        rules = ""

    if container.http:
        config.append(TEMPLATES["NGINX"]["HTTP"].format(
            skyfqdn=container.skyfqdn, name=container.name, dns=NETWORK["dns"],
            port=container.http_port, fqdn=container.fqdn, rules=rules))

    if container.https:
        config.append(TEMPLATES["NGINX"]["HTTPS"].format(
            skyfqdn=container.skyfqdn, name=container.name, dns=NETWORK["dns"],
            port=container.https_port, fqdn=container.fqdn, rules=rules))

    return '\n'.join(config)


def base_commands():
    """Return the base commands in a list"""
    return [cmd for cmd in TEMPLATES["BASE"]["CONTAINERS"].split('\n')]


from __future__ import print_function
from . import __version__
__doc__ = """DryDock v{}

Usage:
    drydock construct [options] <specification>
    drydock deconstruct <specification>
    drydock pull <specification>
    drydock start <specification>
    drydock supervise <specification>
    drydock stop <specification>
    drydock prepare [options]
    drydock master [options] <specification>
    drydock --help | --version

Options:
    -n --dns=<IP>           External dns server to use. [default: 8.8.8.8]
    -d --docker=<IFACE>     Docker  network interface. [default: docker0]
    -l --lan=<IFACE>        Local area network interface. [default: eth0]
    -g --gate=<INT>         Position of network gateway. [default: 1]

    -s --ssl                Generate SSL certificates.

    -h --help               Show this screen.
    -v --version            Show current version.

Commands:
    construct    Construct the given specification.
    deconstruct  Deconstruct the given specification.
    prepare      Prepare docker for constructing a drydock specification.
    master       Create a container that can run a drydock specification.

DryDock takes a simple (YAML) specification file then can construct and
configure a cluster of docker containers. DryDock will automatically setup a
reverse proxy, exposure of ports, and even persistent storage to allow for
easy future upgrading by simply rebuilding the DryDock specification!

For documentation go to http://dry-dock.readthedocs.org/
""".format(__version__)
import yaml
from . import construction
from .duster import MetaContainer
from docopt import docopt
from .templates import prepare_networking


def main():
    """Main entry point."""
    args = docopt(__doc__, version="DryDock v" + __version__)

    if args["construct"]:
        prepare_networking(args["--lan"], args["--docker"], args["--gate"],
                           args["--dns"])
        with open(args["<specification>"], 'r') as drydock:
            construction.construct(
                MetaContainer(**yaml.load(drydock.read())))

    elif args["deconstruct"]:
        with open(args["<specification>"], 'r') as drydock:
            construction.deconstruct(
                MetaContainer(**yaml.load(drydock.read())))

    elif args["pull"]:
        with open(args["<specification>"], 'r') as drydock:
            construction.pull(
                MetaContainer(**yaml.load(drydock.read())))

    elif args["start"]:
        with open(args["<specification>"], 'r') as drydock:
            construction.start(
                MetaContainer(**yaml.load(drydock.read())))

    elif args["supervise"]:
        with open(args["<specification>"], 'r') as drydock:
            construction.supervise(
                MetaContainer(**yaml.load(drydock.read())))

    elif args["stop"]:
        with open(args["<specification>"], 'r') as drydock:
            construction.stop(
                MetaContainer(**yaml.load(drydock.read())))

    elif args["master"]:
        prepare_networking(args["--lan"], args["--docker"], args["--gate"],
                           args["--dns"])
        with open(args["<specification>"], 'r') as drydock:
            construction.master(
                MetaContainer(**yaml.load(drydock.read())),
                args["<specification>"])

    elif args["prepare"]:
        prepare_networking(args["--lan"], args["--docker"], args["--gate"],
                           args["--dns"])
        construction.prepare(args["--ssl"])


if __name__ == "__main__":
    main()



"""
Tests for `drydock.duster` module and Container object.
"""
import pytest
from drydock.duster import Container
from drydock.templates import NETWORK
import yaml


class TestContainer(object):
    config = """
name: blog
base: nekroze/wordpress
domain: nekroze.com
command: crump
envs:
    DB: 123
exposed_ports:
    22: 22
    2222: 222
http_port: 8081
https_port: 4431
external: No
"""

    def test_construction(self):
        container = Container(**yaml.load(self.config))
        assert container.name == "blog"
        assert container.base == "nekroze/wordpress"
        assert container.domain == "nekroze.com"
        assert container.exposed_ports == {22:22, 2222:222}
        assert container.http_port == 8081
        assert container.https_port == 4431
        assert container.external is False
        assert container.skyfqdn == "blog.wordpress.containers.drydock"
        assert container.fqdn == "blog.nekroze.com"

    def test_nginx(self):
        container = Container(**yaml.load(self.config))
        NETWORK["dns"] = "172.17.42.1"
        container.http = True
        container.https = True
        expected = """server {
    listen       80;
    server_name  blog.nekroze.com;

    access_log  /var/log/nginx/blog.nekroze.com.access.log  combined;
    error_log  /var/log/nginx/blog.nekroze.com.error.log;

    resolver 172.17.42.1 valid=5s;
    resolver_timeout 5s;

    location / {
        deny    {gateway};
        allow   {lan}/24;
        allow   {docker}/24;
        deny    all;

        proxy_set_header Host $host;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
        proxy_set_header X-NginX-Proxy true;
        proxy_redirect off;
        proxy_buffering off;

        proxy_pass http://blog.wordpress.containers.drydock:8081/;
    }
}
server {
    listen       443;
    server_name  blog.nekroze.com;

    access_log  /var/log/nginx/blog.nekroze.com.access.log  combined;
    error_log  /var/log/nginx/blog.nekroze.com.error.log;

    resolver 172.17.42.1 valid=5s;
    resolver_timeout 5s;

    ssl on;
    ssl_session_timeout 5m;
    ssl_protocols SSLv2 SSLv3 TLSv1;
    ssl_ciphers ALL:!ADH:!EXPORT56:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv2:+EXP;
    ssl_prefer_server_ciphers on;
    ssl_certificate /etc/nginx/certs/server.crt;
    ssl_certificate_key /etc/nginx/certs/server.key;

    location / {
        deny    {gateway};
        allow   {lan}/24;
        allow   {docker}/24;
        deny    all;

        proxy_set_header Host $host;
        proxy_set_header X-Real-IP $remote_addr;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
        proxy_set_header X-NginX-Proxy true;
        proxy_redirect off;
        proxy_buffering off;

        proxy_pass http://blog.wordpress.containers.drydock:4431/;
    }
}"""
        assert container.get_nginx_config() == expected

    def test_container_commands(self):
        container = Container(**yaml.load(self.config))

        assert container.get_docker_command() == \
               "docker run -d --dns {dockerdns} --name blog -h blog.nekroze.com -p 22:22 -p 2222:222 -v /etc/timezone:/etc/timezone:ro -e \"DB=123\" nekroze/wordpress crump"


"""
Tests for `drydock.duster` module and MetaContainer object.
"""
import pytest
from drydock.duster import MetaContainer
import yaml


class TestMetaContainer(object):
    config = """
name: nekroze.com
domain: nekroze.com

subcontainers:

  - name: blog
    base: nekroze/wordpress
    exposed_ports:
        22: 22
        2222: 222
    http_port: 8081
    https_port: 4431
    external: Yes
    volumes: [/var/lib/mysql]

  - name: root
    base: nekroze/drupal
    http_port: 80
    https_port: 443
"""

    def test_construction(self):
        meta = MetaContainer(**yaml.load(self.config))

        assert meta.name == "nekroze.com"
        assert meta.domain == "nekroze.com"
        assert meta.fqdn == "nekroze.com"

    def test_subcontainer(self):
        meta = MetaContainer(**yaml.load(self.config))

        assert len(meta.containers.keys()) == 2
        assert "blog" in meta.containers.keys()
        assert "root" in meta.containers.keys()

        subcontainer = meta.containers["blog"]

        assert subcontainer.name == "blog"
        assert subcontainer.base == "nekroze/wordpress"
        assert subcontainer.domain == "nekroze.com"
        assert subcontainer.exposed_ports == {22:22, 2222:222}
        assert subcontainer.http_port == 8081
        assert subcontainer.https_port == 4431
        assert subcontainer.external is True
        assert subcontainer.skyfqdn == "blog.wordpress.containers.drydock"
        assert subcontainer.fqdn == "blog.nekroze.com"

    def test_root_subcontainer(self):
        subcontainer = MetaContainer(**yaml.load(self.config)).containers["root"]

        assert subcontainer.domain == "nekroze.com"
        assert subcontainer.fqdn == "nekroze.com"


"""End of run reports."""
__author__ = 'Taylor \"Nekroze\" Lawson'
import sys


class Report(object):
    def __init__(self):
        self.containers = {"success": {}, "failed": {}}
        self.paths = []
        self.commands = {"success": {}, "failed": {}}

    def exit(self):
        """Exit the program with the appropriate code based on the report."""
        if self.containers["failed"] or self.commands["failed"]:
            sys.exit(1)
        else:
            sys.exit(0)

    def container(self, name, command, code):
        """Report on the status of a container"""
        key = "success" if code == 0 else "failed"
        self.containers[key][name] = command

    def path(self, path):
        """Report on the status of a path"""
        self.paths.append(path)

    def command(self, description, command, code):
        """Report on the status of a command"""
        key = "success" if code == 0 else "failed"
        self.commands[key][description] = command

    def render(self, remove=False):
        """
        Render the current state of this report.

        If the ``remove`` argument is true then a removal report will be
        rendered.
        """
        output = ["DryDock Report:"]

        success = len(self.containers["success"].keys())
        failed = len(self.containers["failed"].keys())
        if success:
            output.append("    {} container(s) {}.".format(
                success, "deleted" if remove else "created"))
            output.append("    >" + ', '.join(
                name for name in self.containers["success"].keys()))
        if failed:
            output.append("    {} container(s) failed to be {}.".format(
                failed, "deleted" if remove else "created"))
            for name, command in self.containers["failed"].items():
                output.append("    >{}: {}".format(name, command))

        if success or failed:
            output.append("")

        success = len(self.commands["success"].keys())
        failed = len(self.commands["failed"].keys())
        if success:
            output.append("    {} command(s) succeeded.".format(success))
        if failed:
            output.append("    {} command(s) failed.".format(failed))
            for desc, command in self.commands["failed"].items():
                output.append("    >{}: {}".format(desc, command))

        if success or failed:
            output.append("")

        if len(self.paths):
            output.append("    {} path(s) {}.".format(
                len(self.paths), "deleted" if remove else "written"))
            for path in self.paths:
                output.append("    >" + path)

        return '\n'.join(output)


from __future__ import print_function
__doc__ = """DryDock specification construction functions."""
from os.path import join
import os
import time
import docker
from .templates import base_commands, NETWORK
from .report import Report


def master(specification, filename):
    """Construct the given specification in a master container."""
    report = Report()
    print("\nConstructing drydock master container.")
    fqdn = specification.fqdn

    cmd = ' '.join(["mkdir -p /var/lib/{0}/drydock".format(fqdn)])
    report.command("Create specification store", cmd, os.system(cmd))

    cmd = ' '.join(["cp -f", filename,
                    "/var/lib/{0}/drydock/specification.yaml".format(fqdn)])
    report.command("Store specification", cmd, os.system(cmd))

    master = specification.get_docker_command()
    cmd = ' '.join([master, "dryconstruct"])
    report.container(fqdn, cmd, os.system(cmd))

    cmd = "docker commit --run='{{\"Cmd\": {} }}' {} {}".format(
        '["drysupervise"]', fqdn, fqdn)
    report.command("Run master supervisor", cmd, os.system(cmd))

    cmd = "docker rm " + fqdn
    report.command("Run master supervisor", cmd, os.system(cmd))

    print(report.render())
    print("Master container run command:")
    print(master)
    report.exit()


def prepare(ssl=False):
    report = Report()
    skydns, skydock, nginx, certificate = base_commands()

    print("\nConstructing skydns container.")
    report.container("skydns", skydns, os.system(skydns))

    print("\nConstructing skydock container.")
    report.container("skydock", skydock, os.system(skydock))

    print("\nConstructing nginx container.")
    report.container("nginx", nginx, os.system(nginx))

    if ssl:
        print("\nGenerating SSL/HTTPS certificates for nginx.")
        report.command("Generate SSL Certificates", certificate,
                       os.system(certificate))

    print(report.render())
    report.exit()


def pull(specification):
    """Pull all images for the given specification."""
    report = Report()

    cmd = "docker pull crosbymichael/skydns"
    report.command("Pull image skydns", cmd, os.system(cmd))
    cmd = "docker pull crosbymichael/skydock"
    report.command("Pull image skydns", cmd, os.system(cmd))
    cmd = "docker pull dockerfile/nginx"
    report.command("Pull image nginx", cmd, os.system(cmd))

    for name in set(sorted(specification.containers.keys())):
        base = specification.containers[name].base
        cmd = "docker pull " + base
        report.command("Pull image " + base, cmd, os.system(cmd))

    print(report.render())
    report.exit()


def start(specification):
    """Start all containers defined in the given specification."""
    report = Report()

    cmd = "docker start skydns"
    report.command("Start container skydns", cmd, os.system(cmd))
    cmd = "docker start skydock"
    report.command("Start container skydns", cmd, os.system(cmd))

    for name in sorted(specification.containers.keys()):
        cmd = "docker start " + name
        report.command("Start container " + name, cmd, os.system(cmd))

    cmd = "docker start nginx"
    report.command("Start container nginx", cmd, os.system(cmd))

    print(report.render())
    report.exit()


def supervise(specification):
    """Supervise all containers defined in the given specification."""
    containers = ["skydns", "skydock"]
    containers.extend(sorted(specification.containers.keys()))
    containers.append("nginx")

    dock = docker.Client(base_url='unix://var/run/docker.sock')
    os.system("docker stop nginx && docker rm nginx")
    os.system(base_commands()[2])

    while True:
        ps = {cont["Names"][0][1:]: cont["Status"]
              for cont in dock.containers(all=True)}
        for tag in containers:
            if "Exit" in ps[tag]:  # check for exit status
                print("Container has stopped: {} Code: {}".format(
                    tag, ps[tag][5:]))
                dock.start(tag)
        time.sleep(30)


def stop(specification):
    """Stop all containers defined in the given specification."""
    report = Report()

    cmd = "docker stop skydns"
    report.command("Stop container skydns", cmd, os.system(cmd))
    cmd = "docker stop skydock"
    report.command("Stop container skydns", cmd, os.system(cmd))
    cmd = "docker restart nginx"
    report.command("Stop container nginx", cmd, os.system(cmd))

    for name in sorted(specification.containers.keys()):
        cmd = "docker stop " + name
        report.command("Stop container " + name, cmd, os.system(cmd))

    print(report.render())
    report.exit()


def deconstruct(specification):
    """Deconstruct the given specification."""
    report = Report()

    path = join("/etc/nginx/sites-enabled/", specification.domain)
    if os.path.exists(path):
        os.remove(path)
    report.path(path)

    for name in sorted(specification.containers.keys()):
        print("Stopping and Removing " + name)
        cmd = "docker stop " + name
        report.command("Stop container " + name, cmd, os.system(cmd))

        cmd = "docker rm " + name
        report.container(name, cmd, os.system(cmd))

    print(report.render(True))
    report.exit()


def construct(specification):
    """Construct the given specification."""
    report = Report()
    construct_nginx(specification, report)
    construct_containers(specification, report)

    cmd = "docker stop nginx && docker start nginx"
    report.command("Restart reverse proxy nginx", cmd, os.system(cmd))

    print(report.render())
    report.exit()


def construct_nginx(specification, report):
    """Construct the given specifications nginx configuration file."""
    filename = join("/etc/nginx/sites-enabled/", specification.domain)

    with open(filename, 'w') as site:
            site.write(specification.get_nginx_config())
    report.path(filename)


def construct_containers(specification, report):
    """Run the docker commands to construct each container."""
    for name in sorted(specification.containers.keys()):
        container = specification.containers[name]

        print("\nConstructing " + name)
        cmd = container.get_docker_command()
        cmd.format(dockerdns=NETWORK["dockerdns"])
        report.container(name, cmd, os.system(cmd))



"""DryDock cluster specification."""
from .templates import render_nginx_config, TEMPLATES
from six.moves.urllib.request import urlopen
import yaml


class Container(object):
    """A docker container specification."""
    def __init__(self, name, base="ubuntu", exposed_ports=None, external=True,
                 http_port="80", https_port="443", domain="", volumes=None,
                 envs=None, command=None, data=False):
        self.name = name
        self.domain = ""
        self.fqdn = ""
        self.base = base
        self.envs = envs if envs else {}
        self.data = data
        self.exposed_ports = {}
        self.command = command
        self.http = False
        self.https = False
        if exposed_ports:
            self.exposed_ports.update(exposed_ports)
        self.http_port = http_port
        self.https_port = https_port
        self.external = external
        self.volumes = volumes if volumes else []
        self.set_domain(domain)

    def set_domain(self, domain):
        """Set fqdn and domain."""
        self.domain = domain
        if self.name == "root":
            self.fqdn = self.domain
        else:
            self.fqdn = self.name + '.' + self.domain
        self.skyfqdn = '.'.join([self.name, self.base.split('/')[-1],
                                 "containers", "drydock"])

    def get_portmaps(self):
        """Get docker command port maps."""
        output = []
        for external in sorted(self.exposed_ports.keys()):
            output.append("-p {0}:{1}".format(external,
                                           self.exposed_ports[external]))
        return output

    def get_volumemaps(self):
        """Return docker command volume maps."""
        output = []
        for path in self.volumes:
            output.append("-v /var/lib/{0}/{1}{2}:{2}".format(
                self.domain, self.name, path))
        return output

    def get_envs(self):
        """Return docker environment variable arguments."""
        output = []
        for name, value in self.envs.items():
            output.append("-e \"{0}={1}\"".format(name, value))
        return output

    def get_docker_command(self):
        """Return the docker command required to create this container."""
        cmd = ["docker run -d --dns {dockerdns}"]
        cmd.append("--name " + self.name)
        cmd.append("-h " + self.fqdn)
        cmd.extend(self.get_portmaps())
        if self.data:
            cmd.append("-v /var/lib/{}/drydock/data:/mnt/data".format(
                self.domain))
        cmd.append("-v /etc/timezone:/etc/timezone:ro")
        cmd.extend(self.get_volumemaps())
        cmd.extend(self.get_envs())
        cmd.append(self.base)
        if self.command:
            cmd.append(self.command)
        return ' '.join(cmd)

    def get_nginx_config(self):
        """Get a rendered nginx configuration file for this container."""
        return render_nginx_config(self)


class MetaContainer(Container):
    """A container that stores containers."""
    def __init__(self, name, domain, subcontainers, base="nekroze/drydock", *args,  **kwargs):
        super(MetaContainer, self).__init__(*args, name=name, base=base,
                                            domain=domain, **kwargs)
        if not self.command:
            self.command = \
                "startdocker && drydock supervise /drydock/specification.yaml"
        self.containers = {}
        self.reverse_proxies = {}
        self.fqdn = domain
        self.exposed_ports[self.http_port] = "80"
        self.exposed_ports[self.https_port] = "443"

        for sub in subcontainers:
            if "specification" in sub:
                sub = self.grab_specification(sub["specification"], sub)
            container = Container(**sub)
            container.http = "http_port" in sub
            container.https = "https_port" in sub
            self.add_container(container)

    def grab_specification(self, link, base):
        """Create a container specification based off of a web link."""
        spec = yaml.load(urlopen(link).read()) if link else {}
        spec.update(base)
        del spec["specification"]
        return spec

    def set_domain(self, domain):
        """Set fqdn and domain to the same value."""
        self.domain = domain
        self.fqdn = domain

    def add_container(self, container):
        """Add the given container object to this meta container."""
        if not container.domain and self.domain:
            container.set_domain(self.domain)

        self.containers[container.name] = container
        self.exposed_ports.update(container.exposed_ports)

        fqdn = container.name + '.' + self.domain
        self.reverse_proxies[fqdn] = {
            "skyfqdn": container.skyfqdn,
            "external": container.external,
            "http": container.http_port,
            "https": container.https_port
        }

    def get_nginx_config(self):
        """Get a rendered nginx configuration file for all containers."""
        #output = [TEMPLATES["NGINX"]["HEADER"]]
        output = []
        for name in sorted(self.containers.keys()):
            container = self.containers[name]
            output.append(container.get_nginx_config())
        return '\n'.join(output)

    def get_docker_command(self):
        """
        Return the docker command required to create specification in
        a master container.
        """
        cmd = ["docker run --privileged"]
        cmd.append("--name " + self.fqdn)
        cmd.append("-h " + self.fqdn)
        cmd.extend(self.get_portmaps())
        ngx = "-v /var/lib/{}/nginx/sites-enabled:/etc/nginx/sites-enabled"
        cmd.append(ngx.format(self.fqdn))
        ngx = "-v /var/lib/{}/nginx/ssl:/etc/nginx/ssl:ro"
        cmd.append(ngx.format(self.fqdn))
        cmd.append("-v /etc/timezone:/etc/timezone:ro")
        cmd.append("-v /var/lib/{0}/docker:/var/lib/docker".format(
            self.fqdn))
        cmd.append("-v /var/lib/{0}/drydock/data:/var/lib/{0}/drydock/data".format(
            self.fqdn))
        cmd.append("-v /var/lib/{0}/drydock:/drydock:ro".format(self.fqdn))
        cmd.extend(self.get_volumemaps())
        cmd.append(self.base)
        return ' '.join(cmd)


#!/usr/bin/env python

import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

readme = open('README.rst').read()
doclink = """
Documentation
-------------

The full documentation is at http://dry-dock.readthedocs.org."""

setup(
    name='drydock',
    version='0.5.12',
    description='A Docker cluster construction utility.',
    long_description=readme + '\n\n' + doclink,
    author='Taylor "Nekroze" Lawson',
    author_email='nekroze@eturnilnetwork.com',
    url='https://github.com/Nekroze/drydock',
    packages=[
        'drydock',
    ],
    package_dir={'drydock': 'drydock'},
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'drydock = drydock.drydock:main',
        ]
    },
    install_requires=[
        "pyyaml>=3.10",
        "docopt>=0.6.1",
        "six>=1.5.2",
        "docker-py>=0.3.0",
    ],
    license='MIT',
    zip_safe=False,
    keywords='drydock',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
)
